@@ -15,6 +15,8 @@ class FrameType(Enum):
     BRDM = 'BRDM'
     LOGI = 'LOGI'
     EROR = 'EROR'
+    MEDI = 'MEDI'
+    MRCT = 'MRCT'
 
 
 class FrameModel(SimpleNamespace):
