@@ -2,6 +2,7 @@ from urllib.parse import urlencode
 from .consts import SB_User_Agent, SB_ai
 from .._api.models import Channel, CustomType
 from typing import List
+import logging
 
 
 def get_ws_url(user_id: str, access_token: str):
@@ -30,3 +31,21 @@ def pair_channel_and_names(channels: List[Channel], own_user_id: str):
                     break
         channelid_sub_pairs.update({channel.channel_url: chn_name})
     return channelid_sub_pairs
+
+
+def configure_loggers():
+    logger = logging.getLogger(__name__)
+    logger.setLevel(logging.INFO)
+    logger.propagate = False
+    sh = logging.StreamHandler()
+    sh.setLevel(logging.INFO)
+    sh.setFormatter(logging.Formatter(fmt="%(asctime)s, %(levelname)s: %(message)s", datefmt="%H:%M"))
+    ws_logger = logging.getLogger("websocket")
+    ws_logger.propagate = False
+    ws_logger.addHandler(sh)
+    logger.addHandler(sh)
+    return logger
+
+
+def print_chat_(resp, channelid_sub_pairs):
+    print(f"{resp.user.name}@{channelid_sub_pairs.get(resp.channel_url)}: {resp.message}")
\ No newline at end of file
