@@ -1,6 +1,7 @@
 import requests
 import uuid
-from ._utils.consts import MOBILE_USERAGENT, OAUTH_REDDIT, S_REDDIT, WEB_USERAGENT, OAUTH_CLIENT_ID_B64, WWW_REDDIT, ACCOUNTS_REDDIT
+from ._utils.consts import *
+import abc
 
 
 class _RedditAuthBase:
@@ -14,6 +15,10 @@ class _RedditAuthBase:
     def is_reauthable(self):
         return self._reddit_session is not None and self.api_token is not None
 
+    @abc.abstractmethod
+    def _get_repr_pkl(self):
+        pass
+
     def authenticate(self):
         self._get_userid_sb_token()
         return {'sb_access_token': self.sb_access_token, 'user_id': self.user_id}
@@ -83,7 +88,10 @@ class PasswordAuth(_RedditAuthBase):
             'User-Agent': WEB_USERAGENT,
             'Accept': 'application/json, text/javascript, */*',
             'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
+            'X-Requested-With': 'XMLHttpRequest',
             'Sec-Fetch-Dest': 'empty',
+            'Sec-Fetch-Mode': 'cors',
+            'Sec-Fetch-Site': 'same-origin',
         }
         data = {
             'op': 'login',
@@ -95,7 +103,13 @@ class PasswordAuth(_RedditAuthBase):
         reddit_session = response.cookies.get("reddit_session")
         return reddit_session
 
+    def _get_repr_pkl(self):
+        return self.reddit_username
+
 
 class TokenAuth(_RedditAuthBase):
     def __init__(self, token, reddit_session=None):
         super().__init__(token, reddit_session)
+
+    def _get_repr_pkl(self):
+        return self.api_token
